#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "library.h"
#include "buffer.h"
#include "smalloc.h"
#include "type.h"
#include "object.h"

/* Turn a buffer into an int */
unsigned int
buffer_hash_function(buffer_t *b)
{
	unsigned int hashvalue = 0;
	byte		*cp;
	int			 len;
	int			 i;

	len = buffer_length(b);
	cp = buffer_get(b);

	for (i = 0; i < len; i++)
		hashvalue = (hashvalue << 2) ^ (int)(cp[i]);

	return hashvalue;
}

int
buffer_equals_function(buffer_t *b, buffer_t *s)
{
	return strcmp(buffer_get(b), buffer_get(s));
}

void
buffer_delete(buffer_t *b)
{
	free(b->data);
	object_delete((object_t *)b);
}

void
buffer_init(buffer_t *b)
{
	object_init((object_t *)b);
	b->length = 0;
	b->size = ROPE_LENGTH;
	b->data = (byte *)smalloc(b->size + 1);
}

void
buffer_class_init(buffer_class_t *c)
{
	object_class_t	*oc;
	oc = ((object_class_t *)(c));
	object_class_init(oc);
	oc->delete = (delete_t)buffer_delete;
	oc->hash = (hashfunc_t)buffer_hash_function;
	oc->equals = (compare_t)buffer_equals_function;
}

int
buffer_type(void)
{
	static int	 type = 0;

	if (!type) {
		type_t ti = {
			"buffer",
			sizeof(buffer_t),
			sizeof(buffer_class_t),
			(init_t)buffer_init,
			(cinit_t)buffer_class_init,
			NULL
		};
		type = type_unique(&ti);
	}

	return type;
}

buffer_t *
buffer_new(void)
{
	return (buffer_t *)type_new(buffer_type());
}

byte *
buffer_get(buffer_t *buffer)
{
	return buffer->data;
}

int
buffer_length(buffer_t *buffer)
{
	return buffer->length;
}

static void
buffer_allocate(buffer_t *b, int rq)
{
	int	 newsize;
	newsize = ((b->size > 0)?b->size:ROPE_LENGTH);
	rq++;	/* For terminating NUL in some cases (for rope inherit) */
	while (rq > newsize)
		newsize *= 2;
	if (newsize > b->size) {
		b->data = srealloc(b->data, newsize);
		b->size = newsize;
	}
}

buffer_t *
buffer_add(buffer_t *b, byte *data, int len)
{
	int		 newlen;
	byte	*ptr;

	newlen = b->length + len;
	buffer_allocate(b, newlen);

	/* Can't increment a (void *) - K+R Appendix A7.7 */
	ptr = b->data + b->length;

	/* (void *) is blessed by ANSI and does not require a cast */
	memcpy(ptr, data, len);

	b->length = newlen;
	
	return b;
}

void
buffer_set(buffer_t *b, byte *data, int len)
{
	b->length = 0;
	buffer_add(b, data, len);
}
