#include "library.h"
#include "container.h"
#include "type.h"

static void
container_ref(container_t *c, object_t *o)
{
	object_ref(o);
}

static void
container_unref(container_t *c, object_t *o)
{
#if 0
	DPRINTF((stderr, "About to unref object %p in %p\n", o, c));
#endif
	object_unref(o);
}

void
container_add(container_t *c, object_t *o)
{
#if 0
	DPRINTF((stderr, "Reffing object %s in %s\n",
					object_typeof(o), object_typeof(OBJ(c))));
#endif
	c->add(c, o);
}

void
container_remove(container_t *c, object_t *o)
{
#if 0
	DPRINTF((stderr, "Unreffing object %s in %s\n",
					object_typeof(o), object_typeof(OBJ(c))));
#endif
	c->remove(c, o);
}

void
container_delete(container_t *c)
{
	object_delete((object_t *)c);
}

void
container_init(container_t *c)
{
	object_init((object_t *)c);
	c->add = container_ref;
	c->remove = container_unref;
}

/* The object hash function will do fine */
void
container_class_init(container_class_t *c)
{
	object_class_t	*oc;
	oc = ((object_class_t *)(c));
	object_class_init(oc);
	oc->delete = (delete_t)container_delete;
}

int
container_type(void)
{
	static int	 type = 0;

	if (!type) {
		type_t ti = {
			"container",
			sizeof(container_t),
			sizeof(container_class_t),
			(init_t)container_init,
			(cinit_t)container_class_init,
			NULL
		};
		type = type_unique(&ti);
	}

	return type;
}
