#include <string.h>
#include "library.h"
#include "float.h"
#include "type.h"
#include "smalloc.h"

unsigned int
float_hash_function(float_t *i)
{
	return (int)float_get(i);	/* Oh well. */
}

int
float_equals_function(float_t *f, float_t *g)
{
	float	 h;
	h = float_get(f) - float_get(g);
	if (h < 0)
		return -1;
	else if (h == 0)
		return 0;
	else
		return 1;
}

char *
float_stringify_function(float_t *f)
{
	static char	*buf = NULL;
	static int	 len = 16;	/* Hackish */
	float		 val;

	val = float_get(f);

	if (!buf)
		buf = (char *)smalloc(len * sizeof(char));

	while (len <= snprintf(buf, len, "%f", val)) {
		len *= 2;
		buf = (char *)srealloc(buf, len * sizeof(char));
	}

	return buf;
}

void
float_delete(float_t *f)
{
	object_delete((object_t *)f);
}

void
float_init(float_t *f)
{
	object_init((object_t *)f);
	f->f_value = 0;
}

void
float_class_init(float_class_t *c)
{
	object_class_t	*oc;
	oc = ((object_class_t *)(c));
	object_class_init(oc);
	oc->delete = (delete_t)float_delete;
	oc->hash = (hashfunc_t)float_hash_function;
	oc->equals = (compare_t)float_equals_function;
	oc->string = (stringify_t)float_stringify_function;
}

int
float_type(void)
{
	static int	 type = 0;

	if (!type) {
		type_t ti = {
			"float",
			sizeof(float_t),
			sizeof(float_class_t),
			(init_t)float_init,
			(cinit_t)float_class_init,
			NULL
		};
		type = type_unique(&ti);
	}

	return type;
}

float
float_get(float_t *f)
{
	return f->f_value;
}

void
float_set(float_t *f, float val)
{
	f->f_value = val;
}

float_t *
float_new_f(float val)
{
	float_t	*f;
	f = (float_t *)type_new(float_type());
	float_set(f, val);
	return f;
}
