#ifndef __HASH_H__
#define __HASH_H__

#include "object.h"
#include "vector.h"
#include "container.h"

#define HASH_SIZE		569			// Prime number

typedef delete_t hash_delete_t;

/* This is reasonably private, a one-use structure, not passed around */
typedef
struct _hash_value_t {
	object_t				*key;
	object_t				*data;
	struct _hash_value_t	*next;
} hash_value_t;

typedef
struct _hash_t {
	container_t		  parent;
	int				  size;
	hash_value_t	**data;
	rope_t			 *strbuf;
	hash_delete_t	  h_delete;
} hash_t;

typedef
struct _hash_class_t {
	container_class_t	 parent;
} hash_class_t;

#define hashp(h) (object_typenum(OBJ(h)) == hash_type())

hash_t		*hash_new(void);
int			 hash_type(void);
void		 hash_set_destructor(hash_t *h, hash_delete_t f);
void		 hash_delete(hash_t *hash);
void		 hash_add(hash_t *hashtable, object_t *key, object_t *data);
object_t	*hash_get(hash_t *hashtable, object_t *key);
bool		 hash_remove(hash_t *h, object_t *key);
vector_t	*hash_keys(hash_t *h);

#endif
