#ifndef __VECTOR_H__
#define __VECTOR_H__

#include "library.h"
#include "object.h"
#include "rope.h"
#include "container.h"

#define VECTOR_SIZE	(8)

typedef delete_t vector_delete_t;
typedef object_t *vector_cell_t;
typedef int (*vector_compare_t)(void *, vector_cell_t);

typedef
struct _vector_t {
	container_t		 parent;
	vector_cell_t	*cells;
	int				 size;		/* Allocated */
	int				 length;	/* Used */
	rope_t			*strbuf;	/* Used for stringify() */
	vector_delete_t	 c_delete;
} vector_t;

typedef
struct _vector_class_t {
	container_class_t	 parent;
} vector_class_t;

#define vectorp(v) (object_typenum(OBJ(v)) == vector_type())

vector_t		*vector_new(void);
int				 vector_type(void);
void			 vector_set_destructor(vector_t*v,vector_delete_t func);
int				 vector_length(vector_t *v);
vector_cell_t	 vector_data(vector_t *v, int offset);
void			 vector_delete(vector_t *v);
void			 vector_insert(vector_t *v, vector_cell_t val, int off);
void			 vector_push_back(vector_t *v, vector_cell_t val);
void			 vector_push_front(vector_t *v, vector_cell_t val);
vector_cell_t	 vector_pop(vector_t *v, int off);
vector_cell_t	 vector_pop_front(vector_t *v);
vector_cell_t	 vector_pop_back(vector_t *v);
void			 vector_remove(vector_t *v, int off);
int				 vector_member(vector_t *v, compare_t func, void *val);

#endif
