#include <string.h>
#include "library.h"
#include "integer.h"
#include "type.h"
#include "smalloc.h"

unsigned int
integer_hash_function(integer_t *i)
{
	return abs(integer_get(i));
}

int
integer_equals_function(integer_t *i, integer_t *j)
{
	return integer_get(i) - integer_get(j);
}

char *
integer_stringify_function(integer_t *i)
{
	static char	*buf = NULL;
	static int	 len = 16;	/* Hackish */
	int			 val;

	val = integer_get(i);

	if (!buf)
		buf = (char *)smalloc(len * sizeof(char));

	while (len <= snprintf(buf, len, "%d", val)) {
		len *= 2;
		buf = (char *)srealloc(buf, len * sizeof(char));
	}

	return buf;
}

void
integer_delete(integer_t *i)
{
	object_delete((object_t *)i);
}

void
integer_init(integer_t *i)
{
	object_init((object_t *)i);
	i->i_value = 0;
}

void
integer_class_init(integer_class_t *c)
{
	object_class_t	*oc;
	oc = ((object_class_t *)(c));
	object_class_init(oc);
	oc->delete = (delete_t)integer_delete;
	oc->hash = (hashfunc_t)integer_hash_function;
	oc->equals = (compare_t)integer_equals_function;
	oc->string = (stringify_t)integer_stringify_function;
}

int
integer_type(void)
{
	static int	 type = 0;

	if (!type) {
		type_t ti = {
			"integer",
			sizeof(integer_t),
			sizeof(integer_class_t),
			(init_t)integer_init,
			(cinit_t)integer_class_init,
			NULL
		};
		type = type_unique(&ti);
	}

	return type;
}

int
integer_get(integer_t *i)
{
	return i->i_value;
}

void
integer_set(integer_t *i, int val)
{
	i->i_value = val;
}

integer_t *
integer_new_i(int val)
{
	integer_t	*i;
	i = (integer_t *)type_new(integer_type());
	integer_set(i, val);
	return i;
}
