#include <string.h>
#include "library.h"
#include "object.h"
#include "type.h"

void object_init(object_t *o);
void object_class_init(object_class_t *c);
void object_delete(object_t *o);

unsigned int
object_hash_function(object_t *o)
{
	return (unsigned int)(o);	/* Will this break things? */
}

int
object_compare_function(object_t *o, object_t *p)
{
#if 0
	/* Beware what ANSI does and does not define for pointers */
	if (o == p)
		return 0;
	return 1;
#else
	/* Fuckit */
	return o - p;
#endif
}

char *
object_stringify_function(object_t *o)
{
	return "(null)";
}

int
object_type(void)
{
	static int	 type = 0;

	if (!type) {
		type_t ti = {
				"object",
				sizeof(object_t),
				sizeof(object_class_t),
				(init_t)object_init,
				(cinit_t)object_class_init,
				NULL
			};
		type = type_unique(&ti);
	}

	return type;
}

void
object_init(object_t *o)
{
}

void
object_class_init(object_class_t *c)
{
	c->delete = (delete_t)object_delete;
	c->hash = (hashfunc_t)object_hash_function;
	c->equals = (compare_t)object_compare_function;
	c->string = (stringify_t)object_stringify_function;
}

void
object_delete(object_t *o)
{
}

void
object_ref(object_t *o)
{
	type_ref(o);
}

void
object_unref(object_t *o)
{
	type_unref(o);
}

/* Generic object methods */

object_class_t *
object_class(object_t *o)
{
	return o->class;
}

int
object_typenum(object_t *o)
{
	object_class_t	*c;
	c = object_class(o);
	return c->type;
}

char *
object_typeof(object_t *o)
{
	return type_name(object_typenum(o));
}

/* 0 for equality, 1 for inequality */
int
object_equals(object_t *o, object_t *p)
{
	object_class_t	*c;
	object_class_t	*d;

	c = object_class(o);
	d = object_class(p);

	// DPRINTF((stderr, "Got two objects %p, %p\n", o, p));
	// DPRINTF((stderr, "Got two classes %p, %p\n", c, d));

	if (c->type != d->type)
		return 1;
	return c->equals(o, p);
}

unsigned int
object_hash(object_t *o)
{
	object_class_t	*c;
	c = object_class(o);
	return c->hash(o);
}

char *
object_stringify(object_t *o)
{
	object_class_t	*c;
	c = object_class(o);
	return c->string(o);
}
