#include <string.h>
#include "library.h"
#include "pointer.h"
#include "type.h"
#include "smalloc.h"

unsigned int
pointer_hash_function(pointer_t *p)
{
	/* This is K+R, if an unsigned long has room for a (void *),
	 * but the conversion is system dependent. 65537 is prime. */
	return (((unsigned long)pointer_get(p)) % 65537);
}

int
pointer_equals_function(pointer_t *p, pointer_t *q)
{
	/* XXX Not quite K+R! */
	return pointer_get(p) - pointer_get(q);
}

char *
pointer_stringify_function(pointer_t *p)
{
	static char	*buf = NULL;
	static int	 len = 16;	/* Hackish */
	void		*val;

	val = pointer_get(p);

	if (!buf)
		buf = (char *)smalloc(len * sizeof(char));

	while (len <= snprintf(buf, len, "%p", val)) {
		len *= 2;
		buf = (char *)srealloc(buf, len * sizeof(char));
	}

	return buf;
}

void
pointer_delete(pointer_t *p)
{
	if (p->pdelete)
		p->pdelete(p->data);
	object_delete((object_t *)p);
}

void
pointer_init(pointer_t *p)
{
	object_init((object_t *)p);
	p->data = NULL;
}

void
pointer_class_init(pointer_class_t *c)
{
	object_class_t	*oc;
	oc = ((object_class_t *)(c));
	object_class_init(oc);
	oc->delete = (delete_t)pointer_delete;
	oc->hash = (hashfunc_t)pointer_hash_function;
	oc->equals = (compare_t)pointer_equals_function;
	oc->string = (stringify_t)pointer_stringify_function;
}

int
pointer_type(void)
{
	static int	 type = 0;

	if (!type) {
		type_t ti = {
			"pointer",
			sizeof(pointer_t),
			sizeof(pointer_class_t),
			(init_t)pointer_init,
			(cinit_t)pointer_class_init,
			NULL
		};
		type = type_unique(&ti);
	}

	return type;
}

void *
pointer_get(pointer_t *p)
{
	return p->data;
}

void
pointer_set(pointer_t *p, void *data)
{
	p->data = data;
}

void
pointer_set_destructor(pointer_t *p, delete_t func)
{
	p->pdelete = func;
}

pointer_t *
pointer_new_Pv(void *data)
{
	pointer_t	*p;
	p = (pointer_t *)type_new(pointer_type());
	pointer_set(p, data);
	return p;
}

pointer_t *
pointer_new_PvP8delete_t(void *data, delete_t func)
{
	pointer_t	*p;
	p = (pointer_t *)type_new(pointer_type());
	pointer_set(p, data);
	pointer_set_destructor(p, func);
	return p;
}
