#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "library.h"
#include "rope.h"
#include "smalloc.h"
#include "type.h"
#include "object.h"
#include "encode.h"

/* This class should really inherit buffer, but it's not an
 * immediate concern - 6 hours to go. */

/* Turn a rope into an int */
unsigned int
rope_hash_function(rope_t *r)
{
	unsigned int hashvalue = 0;
	char *cp;

	for (cp = rope_get(r); *cp; cp++)
		hashvalue = (hashvalue << 2) ^ (int)(*cp);

	return hashvalue;
}

int
rope_equals_function(rope_t *r, rope_t *s)
{
	return strcmp(rope_get(r), rope_get(s));
}

char *
rope_stringify_function(rope_t *r)
{
	static rope_t	*s;
	if (!s) {
		s = rope_new();
		object_ref(OBJ(s));	/* Static */
	}

	rope_set(s, "\"");
	rope_add(s, encode_i3(rope_get(r)));
	rope_add(s, "\"");
	return rope_get(s);
}

void
rope_delete(rope_t *r)
{
	free(r->chars);
	object_delete((object_t *)r);
}

void
rope_init(rope_t *r)
{
	object_init((object_t *)r);
	r->length = 0;
	r->size = ROPE_LENGTH;
	r->chars = smalloc(r->size + 1);
}

void
rope_class_init(rope_t *c)
{
	object_class_t	*oc;
	oc = ((object_class_t *)(c));
	object_class_init(oc);
	oc->delete = (delete_t)rope_delete;
	oc->hash = (hashfunc_t)rope_hash_function;
	oc->equals = (compare_t)rope_equals_function;
	oc->string = (stringify_t)rope_stringify_function;
}

int
rope_type(void)
{
	static int	 type = 0;

	if (!type) {
		type_t ti = {
			"rope",
			sizeof(rope_t),
			sizeof(rope_class_t),
			(init_t)rope_init,
			(cinit_t)rope_class_init,
			NULL
		};
		type = type_unique(&ti);
	}

	return type;
}

rope_t *
rope_new(void)
{
	return (rope_t *)type_new(rope_type());
}

char *
rope_get(rope_t *rope)
{
	return rope->chars;
}

int
rope_length(rope_t *rope)
{
	return rope->length;
}

static void
rope_allocate(rope_t *r, int rq)
{
	int	 newsize;
	newsize = ((r->size > 0)?r->size:ROPE_LENGTH);
	rq++;	/* For terminating NUL */
	while (rq > newsize)
		newsize *= 2;
	if (newsize > r->size) {
		r->chars = (char *)srealloc(r->chars,
						(newsize * sizeof(char)));
		r->size = newsize;
	}
}

rope_t *
rope_add(rope_t *targ, char *src)
{
	int	 len;

	len = targ->length + strlen(src);
	rope_allocate(targ, len);

	strcpy(&(targ->chars[targ->length]), src);
	targ->length = len;
	
	return targ;
}

rope_t *
rope_concat(rope_t *targ, rope_t *src)
{
	return rope_add(targ, src->chars);
}

void
rope_set(rope_t *targ, char *str)
{
	targ->length = 0;
	rope_add(targ, str);
}

char *
rope_substr(rope_t *rope, int start, int end)
{
	int		 len;
	char	*str;
	

	if (start < 0)
		start = 0;
	if (end > rope->length)
		end = rope->length;
	if (end < start)
		end = start;
	
	len = end - start;
	str = (char *)smalloc((len + 1) * sizeof(char));
	strncpy(str, &(rope->chars[start]), len);
	str[len] = '\0';
	return str;
}

void
rope_replace(rope_t *rope, char *old, char *new)
{
}

/* Constructors */
rope_t *
rope_new_Pc(char *str)
{
	rope_t	*r;
	r = (rope_t *)type_new(rope_type());
	rope_set(r, str);
	return r;
}
