#include <string.h>
#include "smalloc.h"
#include "type.h"

/*
 * type data
 * make class
 * init class
 * return class to client
 */

static type_t			*types = NULL;
static int				 types_nalloc = 8;
static int				 types_nused = 0;

static void
type_allocate(void)
{
	if ((!types) || (types_nused >= types_nalloc)) {
		types_nalloc *= 2;
		types = (type_t *)srealloc(types, types_nalloc * sizeof(type_t));
	}
}

/* Note, we don't use slot 0 */
int
type_unique(type_t *typeinfo)
{
	type_t	*new;
	types_nused++;
	type_allocate();
	new = &types[types_nused];
	memcpy(new, typeinfo, sizeof(type_t));
	new->class = (object_class_t *)smalloc(new->csize);
	memset(new->class, 0, new->csize);
	new->class->type = types_nused;
	new->cinit(new->class);
	// DPRINTF((stderr, "Made new type \"%s\" (%d)\n", typeinfo->name, types_nused));
	return types_nused;
}

object_class_t *
type_class(int type)
{
#if 0
	DPRINTF((stderr, "Class pointer for type %s (%d) is %p\n",
					types[type].name, type, types[type].class));
#endif
	return types[type].class;
}

char *
type_name(int type)
{
	return types[type].name;
}

object_t *
type_new(int type)
{
	type_t		*t;
	object_t	*o;

	t = &types[type];
	o = (object_t *)smalloc(t->size);
	memset(o, 0, t->size);
	o->class = type_class(type);
	t->init(o);
	return o;
}

void
type_delete(object_t *o)
{
	object_class_t	*c;
	c = o->class;
	if (c->delete)
		c->delete(o);
	sfree(o);
}

int
type_refs(object_t *o)
{
	return o->refcount;
}

void
type_ref(object_t *o)
{
	// DPRINTF((stderr, "Reffed %s\n", object_typeof(o)));
	o->refcount++;
}

void
type_unref(object_t *o)
{
	o->refcount--;
	if (o->refcount <= 0) {
		if (o->refcount < 0)
			DPRINTF((stderr, "Unreffed %s below zero\n", object_typeof(o)));
#if 0
		else
			DPRINTF((stderr, "Unreffed %s to zero\n", object_typeof(o)));
#endif
		type_delete(o);
	}
}
