#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "list.h"
#include "smalloc.h"

static list_item_t *freenode = NULL;

static list_item_t *list_item_new(char *label, list_data_t data)
{
    list_item_t *item;
   
    if (freenode)
    {
        item = freenode;
        freenode = freenode->next;
    }
    else
        item = (list_item_t *)smalloc(sizeof(list_item_t));

    
    memset(item, 0, sizeof(item));
    if (label)
        item->label = sstrdup(label);
    
    item->data = data;
    
    return item;
}

/* Data removed in data_delete() */
static void list_item_delete(list_item_t *item)
{
    free(item->label);
    item->next = freenode;
    freenode = item;
}

list_t *
list_new(list_delete_t func)
{
   list_t *list;
   list_item_t *node;
   
   list = (list_t *)smalloc(sizeof(list_t));
   list->length = 0;
   
   node = list_item_new(NULL, NULL);
   node->prev = node;
   node->next = node;
   
   list->root = node;
   
   list->i_delete = func;
   
   return list;
}

int
list_length(list_t *list)
{
   return list->length;
}

void
list_data_insert(list_iterator_t *iterator,
				char *label, list_data_t data)
{
   list_item_t *item;
   list_item_t *where;
   
   item = list_item_new(label, data);
   
   where = iterator->node;
   
   item->next = where;
   item->prev = where->prev;
   where->prev->next = item;
   where->prev = item;

   iterator->list->length++;
}

void
list_data_remove(list_iterator_t *iterator)
{
   list_item_t *item;
   list_t *list;
   
   item = iterator->node;
   list = iterator->list;
   
   list_iterator_prev(iterator);
   
   item->next->prev = item->prev;
   item->prev->next = item->next;
   
   list->length--;
   
   if (list->i_delete)
      list->i_delete(item->data);
   
   list_item_delete(item);
   
}

list_data_t
list_data_read_data(list_iterator_t *iterator)
{
   return iterator->node->data;
}

char *
list_data_read_label(list_iterator_t *iterator)
{
   return iterator->node->label;
}

bool
list_iterator_begin(list_iterator_t *iterator)
{
   list_t *list;
   
   list = iterator->list;
   
   iterator->node = list->root->next;
   
   return (list->root != list->root->next);
}

bool
list_iterator_end(list_iterator_t *iterator)
{
   iterator->node = iterator->list->root;
   return FALSE;
}

bool
list_iterator_prev(list_iterator_t *iterator)
{
   list_item_t *item;
   
   item = iterator->node->prev;
   
   iterator->node = item;
   
   return (item != iterator->list->root);
}

bool
list_iterator_next(list_iterator_t *iterator)
{
   list_item_t *item;
   
   item = iterator->node->next;
   
   iterator->node = item;
   
   return (item != iterator->list->root);
}

list_iterator_t *
list_iterator_new(list_t *list)
{
   list_iterator_t *iterator;
   
   iterator = (list_iterator_t *)smalloc(sizeof(list_iterator_t));
   
   iterator->list = list;
   
   list_iterator_end(iterator);
   
   return iterator;
}

void
list_iterator_delete(list_iterator_t *iterator)
{
   free(iterator);
}

void
list_delete(list_t *list)
{
	list_iterator_t *iterator;

	iterator = list_iterator_new(list);

	list_iterator_end(iterator);	/* So next gives begin */

	while (list_iterator_next(iterator))
		list_data_remove(iterator);

	list_iterator_delete(iterator);

	list_item_delete(list->root);

	free(list);
}
