#include <stdlib.h>
#include <regex.h>
#include <ctype.h>
#include <stdio.h>
#include "library.h"
#include "smalloc.h"
#include "wregex.h"

static char *errbuf = NULL;
static bool	 error = FALSE;

wregex_t *
wregex_new (char *str, int flags)
{
	regex_t	*preg;
	int		 re_flags;
	int		 re_error;
	int		 errsize;

	preg = (regex_t *)smalloc(sizeof(regex_t));
	re_flags = (REG_NOSUB | flags);
	// Don't need REG_NEWLINE
	// Removed REG_ICASE as everything is lowercase now
	re_error = regcomp(preg, str, re_flags);
	if (re_error) {
		errsize = (regerror(re_error, preg, NULL, (size_t)(0)) +1);
		errbuf = srealloc(errbuf, errsize);
		regerror (re_error, preg, errbuf, errsize);
		error = TRUE;
		fprintf(stderr, "HEY! Get the regex error\n");
		regfree(preg);
		sfree(preg);
		return NULL;	/* Does this leak? */
	}
	error = FALSE;
	return (wregex_t *)(preg);
}

void
wregex_delete (wregex_t *preg)
{
	regfree((regex_t *)(preg));
	sfree(preg);
}

/* 0 on match, 1 otherwise */
bool
wregex_execute (wregex_t *preg, char *str)
{
	return !!regexec((regex_t *)(preg), str, (size_t)(0), NULL, 0);
}

char *
wregex_errorstr(void)
{
	return errbuf;
}

bool
wregex_error(void)
{
	return error;
}
