#include <stdio.h>
#include <stdarg.h>
#include <assert.h>
#include "lists.h"
#include "value.h"

value *
array_new (void)
{
	value *v = value_new(MM_ARRAY);
	v->v_plength = 4;	/* strlen("({})") */
	v->v_array = list_new();
	return v;
}

value *
array_append (value *a, value *v)
{
	listentry	*node;
	assert(a);
	assert(a->v_type == MM_ARRAY);
	node = list_item_new();
	node->data = (void *)(v);
	list_append(a->v_array, node);
	a->v_plength += (v?v->v_plength:V_LEN_NUL) + V_LEN_COMMA;
	return a;
}

#define A_NSTR(a, s) array_append(a, value_new_data(MM_STRING, s))
#define A_NVAL(a, v) array_append(a, value_new_data(MM_VALUE, (void*)v))
#define A_NARR(a, d) array_append(a, d)
#define A_NMAP(a, d) array_append(a, d)
#define A_NNON(a)    array_append(a, value_new(MM_NONE));

value *
array_create (value *a, char *fmt, ... )
{
	va_list	 ap;
	va_start(ap, fmt);
	if (!a) a = array_new();
	while (*fmt) {
		switch(*fmt++) {
			case 's':
				A_NSTR(a, va_arg(ap, char *));
				break;
			case 'v':
				A_NVAL(a, va_arg(ap, long));
				break;
			case 'a':
				A_NARR(a, va_arg(ap, value *));
				break;
			case 'm':
				A_NMAP(a, va_arg(ap, value *));
				break;
			case 'n':
				A_NNON(a);
				break;
			default:
				fprintf(stderr, "Error in format string at '%c'",
								*(fmt-1));
				break;
		}
	}
	return a;
}
