%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "value.h"
#include "mudmode.h"
#include "lists.h"
#include "tree.h"
#include "array.h"
#include "mapping.h"
%}

%option noyywrap
%option always-interactive
%pointer

%x ERROR

dDIGIT	[[:digit:]]
dCHAR	[[:alpha:]]
dWHITE	[[:blank:]]
dMUDCHAR	[^\n\"]
dSTRING		\"{dMUDCHAR}*\"
dVALUE		{dDIGIT}+
dSPACE		{dWHITE}+
dARRAY		\(\{
dEARRAY		\}\)
dMAPPING	\(\[
dEMAPPING	\]\)
dCOMMA		,
dCOLON		:
dEOL		\n

%%

{dSPACE}	{ }
{dARRAY}	{ return(MM_ARRAY); }
{dEARRAY}	{ return(MM_EARRAY); }
{dMAPPING}	{ return(MM_MAPPING); }
{dEMAPPING}	{ return(MM_EMAPPING); }
{dSTRING}	{ return(MM_STRING); }
{dVALUE}	{ return(MM_VALUE); }
{dEOL}		{ return(MM_EOL); }
{dCOMMA}	{ return(MM_COMMA); }
{dCOLON}	{ return(MM_COLON); }
<ERROR>.*$	{ BEGIN(INITIAL); return(MM_NONE); }
<*><<EOF>>	{ return(MM_EOF); }
.			{ return(MM_ERR); }

%%

int			 yylval;
int			 yyerror;

int
parse_expect (int vtype)
{
	yylval = yylex();
	if (!(yylval & vtype)) {
		fprintf(stderr, "Syntax error before '%s'\n", yytext);
		fprintf(stderr, "Expected type %d got %d\n", vtype, yylval);
		BEGIN(ERROR);
		yylex();
		yyerror = 1;
		return 1;
	}
	return 0;
}

value * parse_value_array (value *a);
value * parse_value_mapping (value *m);

value *
parse_value (void)
{
	value		*v = NULL;
	char		*cp;

	yylval = yylex();
	switch(yylval) {
		case MM_ARRAY:
			/* Create new array node pointing to array */
			/* parse_value_array() */
			// v = value_new_data(MM_ARRAY, parse_value_array());
			v = array_new();
			parse_value_array(v);
			break;
		case MM_MAPPING:
			// v = value_new_data(MM_MAPPING, parse_value_mapping(v));
			v = mapping_new();
			parse_value_mapping(v);
			break;
		case MM_STRING: /* Create string item */
			while ((cp = strchr(yytext, '\r'))) *cp = '\n';
			v = value_new_data(MM_STRING, yytext);
			// v->v_value = yyleng;
			break;
		case MM_VALUE: /* Create value item */
			v = value_new(MM_VALUE);
			v->v_value = atol(yytext);
			break;

		case MM_COMMA: /* Might as well hande NULLs */
			fprintf(stderr, "Unexpected comma, assuming NULL."
				" before ',' DEIMPLEMENTED\n");
			unput(',');	/* Put the expected comma back */
			break;
		case MM_EOL:
			/* Check stack, terminate arrays */
			break;
		case MM_EOF:
			/* Clear stack, exit cleanly */
			break;
		case MM_EARRAY: /* Got null value again */
			fprintf(stderr, "Unexpected end of array.\n");
			unput(')'); unput('}');
			break;
		case MM_EMAPPING:
			fprintf(stderr, "Unexpected end of mapping\n");
			unput(')'); unput(']');
			break;
		case MM_ERR:
			/* Syntax error, sort this out */
			unput(*yytext);
			BEGIN(ERROR);
			yylval = yylex();
			fprintf(stderr, "Syntax error before '%.8s'", yytext);
			if (yyleng > 8) fprintf(stderr, "...");
			fprintf(stderr, "\n");
			break;
	}

	return v;
}

value *
parse_value_mapping (value *m)
{
	value		*k;
	value		*d;

	while((k = parse_value())) {

		if (yyerror) {
			mapping_append(m, mapping_data_new(k, NULL));
			return m;
		}

		if (parse_expect(MM_COLON)) {
			mapping_append(m, mapping_data_new(k, NULL));
			return m;
		}

		d = parse_value();
		if (!d) break;
		mapping_append(m, mapping_data_new(k, d));
		if (yyerror) return m;

		if (parse_expect(MM_EMAPPING|MM_COMMA)) return m;
		if (yylval == MM_EMAPPING) return m;
	}

	fprintf(stderr, "Syntax error at end of input\n");
	return m;
}

value *
parse_value_array (value *a)
{
	value		*v;

	while ((v = parse_value())) {
		array_append(a, v);
		if (yyerror) return a;

		if (parse_expect(MM_EARRAY|MM_COMMA)) return a;
		if (yylval == MM_EARRAY) return a;
	}
	fprintf(stderr, "Syntax error at end of input\n");
	return a;
}

value *
mudmode_main (FILE *f)
{
	value	*tree;
	yyin = f;
	yyerror = 0;
	yylval = MM_NONE;
	while (!(yylval&(MM_EOF|MM_ERR))) {
		tree = parse_value();
		if (!(yylval&(MM_EOF|MM_EOL|MM_ERR))) parse_expect(MM_EOL);
		// if (tree) tree_print(1, tree);
		else printf("Null input\n");
	}
	return tree;
}
