#include <stdio.h>
#include <netinet/in.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "value.h"
#include "lists.h"
#include "tree.h"

static char	*tree_buf = NULL;
static int	 tree_length = 0;
static int	 tree_offset = 0;

#define BUFFER_ADD(x) do { int len = strlen(x);				\
				strncpy(&tree_buf[tree_offset], x, len);	\
				tree_offset += len; } while(0);

void
tree_allocate(int length)
{
	tree_offset = 0;
	if (length >= tree_length) {
		tree_buf = (char *)realloc(tree_buf,
						(length + 10) * sizeof(char));
		tree_length = length + 10;
	}
}

void tree_print_array (value *v);
void tree_print_mapping (value *v);

void
tree_print_value (value *v)
{
	if (!v) {
		BUFFER_ADD(VNUL);
		return;
	}
	switch(v->v_type) {
		case MM_ARRAY:
			BUFFER_ADD("({");
			tree_print_array(v);
			BUFFER_ADD("})");
			break;
		case MM_MAPPING:
			BUFFER_ADD("([");
			tree_print_mapping(v);
			BUFFER_ADD("])");
			break;
		case MM_STRING:
			if (v->v_string) {
				BUFFER_ADD("\"");
				BUFFER_ADD(v->v_string);
				BUFFER_ADD("\"");
			}
			else {
				BUFFER_ADD("0");
			}
			break;
		case MM_VALUE:
			sprintf(&tree_buf[tree_offset], "%ld", v->v_value);
			tree_offset += v->v_plength;
			break;
		case MM_NONE:
			// fprintf(stream, "[ERROR]");
			/* This allows just to print a comma blank */
			break;
	}
}

void
tree_print_array (value *v)
{
	listentry	*list;
	listentry	*node;

	if (!v) return;
	list = (listentry *)(v->v_array);	/* list asserted above */
	node = list->next;
	while (node != list) {
		tree_print_value((value *)(node->data));
		BUFFER_ADD(",");
		node = node->next;
	}
}

void
tree_print_mapping (value *v)
{
	listentry	*list;
	listentry	*node;
	value		*d;

	list = (listentry *)(v->v_array);	/* list asserted above */
	node = list->next;
	while (node != list) {
		d = (value *)(node->data);
		tree_print_value(d->k);
		BUFFER_ADD(":");
		tree_print_value(d->d);
		BUFFER_ADD(",");
		node = node->next;
	}
}

char *
tree_print (value *v)
{
	int	 nlen;

	tree_allocate(v->v_plength + 4);

	nlen = htonl(v->v_plength);
	memcpy(tree_buf, &nlen, sizeof(int));
	tree_offset += sizeof(int);

	tree_print_value(v);
	tree_buf[tree_offset] = 0;	/* XXX For convenience */

	return tree_buf;
}
