#include "library.h"
#include "integer.h"
#include "float.h"
#include "object.h"
#include "type.h"
#include "rope.h"
#include "vector.h"
#include "utils.h"
#include "hash.h"

void
test_equality(object_t *o, object_t *p, int t)
{
	int				 i;
	object_class_t	*c;
	object_class_t	*d;

	c = object_class(o);
	d = object_class(p);

	printf("Testing equality of a %s and a %s: ",
					type_name(c->type),
					type_name(d->type));
	i = object_equals(o, p);
	if (i == 0)
		printf("The two are equal\n");
	else
		printf("The two are not equal\n");

	if ((!!i) != (!!t))
		printf("ERROR! Test fails\n");
}

object_t *
test_float(void)
{
	float_t	*f;
	float_t	*g;
	float_t	*h;

	f = float_new_f(8.213);
	object_ref(OBJ(f));
	printf("Created new float f <%p> value %f\n", f, float_get(f));

	g = float_new_f(8.213);
	object_ref(OBJ(g));
	printf("Created new float g <%p> value %f\n", g, float_get(g));

	h = float_new_f(17.345);
	object_ref(OBJ(h));
	printf("Created new float h <%p> value %f\n", h, float_get(h));

	test_equality(OBJ(f), OBJ(g), 0);
	test_equality(OBJ(f), OBJ(h), 1);

	object_unref(OBJ(f));
	object_unref(OBJ(h));
	return OBJ(g);
}

object_t *
test_integer(void)
{
	integer_t	*i;
	integer_t	*j;
	integer_t	*k;

	i = integer_new_i(17);
	printf("Created new integer i <%p> value %d\n", i, integer_get(i));
	j = integer_new_i(17);
	printf("Created new integer j <%p> value %d\n", j, integer_get(j));
	k = integer_new_i(34);
	printf("Created new integer k <%p> value %d\n", k, integer_get(k));

	object_ref(OBJ(i));
	object_ref(OBJ(j));
	object_ref(OBJ(k));

	test_equality(OBJ(i), OBJ(j), 0);
	test_equality(OBJ(i), OBJ(k), 1);

	object_unref(OBJ(i));
	object_unref(OBJ(k));
	return OBJ(j);
}

object_t *
test_rope(void)
{
	rope_t	*r;
	rope_t	*s;
	rope_t	*t;

	r = rope_new_Pc("foo");
	printf("Created new rope r <%p> value %s\n", r, rope_get(r));
	s = rope_new_Pc("foo");
	printf("Created new rope s <%p> value %s\n", s, rope_get(s));
	t = rope_new_Pc("bar");
	printf("Created new rope t <%p> value %s\n", t, rope_get(t));

	object_ref(OBJ(r));
	object_ref(OBJ(s));
	object_ref(OBJ(t));

	test_equality(OBJ(r), OBJ(s), 0);
	test_equality(OBJ(r), OBJ(t), 1);

	printf("Adding %s to %s\n", "baz", rope_get(s));
	rope_add(s, "baz");
	printf("Rope is now %s\n", rope_get(s));

	test_equality(OBJ(r), OBJ(s), 0);

	object_unref(OBJ(r));
	object_unref(OBJ(t));
	return OBJ(s);
}

object_t *
test_vector(void)
{
	vector_t	*v;

	v = vector_new();
	object_ref(OBJ(v));

	vector_push_back(v, OBJ(rope_new_Pc("0")));
	printf("Push back;\tVector is %s\n", rope_get(implode(v, ", ")));
	vector_push_back(v, OBJ(rope_new_Pc("1")));
	printf("Push back;\tVector is %s\n", rope_get(implode(v, ", ")));
	vector_insert(v, OBJ(rope_new_Pc("2")), 1);
	printf("Insert 1;\tVector is %s\n", rope_get(implode(v, ", ")));
	vector_push_front(v, OBJ(rope_new_Pc("3")));
	printf("Push front;\tVector is %s\n", rope_get(implode(v, ", ")));
	vector_insert(v, OBJ(rope_new_Pc("4")), 2);
	printf("Insert 2;\tVector is %s\n", rope_get(implode(v, ", ")));
	vector_pop(v, 3);
	printf("Pop 3;   \tVector is %s\n", rope_get(implode(v, ", ")));
	vector_pop_front(v);
	printf("Pop front;\tVector is %s\n", rope_get(implode(v, ", ")));
	vector_pop_back(v);
	printf("Pop back;\tVector is %s\n", rope_get(implode(v, ", ")));
	vector_remove(v, 0);
	printf("Remove 0;\tVector is %s\n", rope_get(implode(v, ", ")));

	return OBJ(v);
}

#define STR "||a|aa|aaa||a|aaa|a|ok|||||Sumgait Caspian Sea|"

void
test_explode(void)
{
	vector_t	*v;
	rope_t		*s;
	rope_t		*t;

	s = rope_new_Pc(STR);
	v = explode(s, "|");
	t = implode(v, "|");

	object_ref(OBJ(s));
	object_ref(OBJ(v));
	object_ref(OBJ(t));

	printf("Line is %s\n", STR);
	printf("Rope  is %s\n", rope_get(s));
	printf("Str  is %s\n", rope_get(t));

	test_equality(OBJ(s), OBJ(t), 0);

	object_unref(OBJ(s));
	object_unref(OBJ(v));
	object_unref(OBJ(t));
}

void
test_hash(void)
{
	hash_t		*h;
	object_t	*r;
	object_t	*s;
	object_t	*t;

	h = (hash_t *)type_new(hash_type());
	object_ref(OBJ(h));

	r = OBJ(rope_new_Pc("foo"));
	s = OBJ(rope_new_Pc("foo - bar"));
	hash_add(h, r, s);

	r = OBJ(rope_new_Pc("bar"));
	s = OBJ(rope_new_Pc("bar - bar"));
	hash_add(h, r, s);

	t = hash_get(h, r);
	printf("Got %s from hash table\n", rope_get((rope_t *)t));
	test_equality(s, t, 0);

	r = OBJ(rope_new_Pc("foo"));
	object_ref(OBJ(r));
	t = hash_get(h, r);
	object_unref(OBJ(r));
	printf("Got %s from hash table\n", rope_get((rope_t *)t));
	test_equality(s, t, 1);

	r = OBJ(integer_new_i(19));
	object_ref(r);

	hash_add(h, r, t);
	t = hash_get(h, r);
	printf("Got %s from hash table\n", rope_get((rope_t *)t));
	test_equality(s, t, 1);

	printf("Removing entry %s from hash\n", object_stringify(r));
	hash_remove(h, r);
	printf("Removed entry\n");
	t = hash_get(h, r);
	printf("Attempted to get entry\n");
	if (t)
		printf("ERROR: Got entry from hashtable after removal\n");
	else
		printf("Remove succeeded\n");
	object_unref(r);

	object_unref(OBJ(h));
}

int
main(int argc, char **argv)
{
	object_t	*a;
	object_t	*b;
	object_t	*c;
	object_t	*d;

	a = test_float();
	b = test_integer();
	c = test_rope();
	d = test_vector();

	test_equality(a, b, 1);
	test_equality(b, c, 1);
	test_equality(a, c, 1);
	test_equality(a, d, 1);
	test_equality(d, d, 0);

	object_unref(a);
	object_unref(b);
	object_unref(c);
	object_unref(d);

	test_explode();
	test_hash();

	return 0;
}
